# %W% %G% %U% - %Q% #
#ident "@(#)VCSDB:%M% %I%"
#
# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
 
package db2lib;
use strict;
use ag_i18n_inc;
use File::Basename;
my ($argResourceName) = @ARGV;
VCSAG_SET_ENVS ($argResourceName, 20004);

use lib qw(.);
use db2config () ;
use vars qw(%CONF);

*CONF =\%db2config::CONF;
my $CAT = $CONF{CAT};
my $SU = $CONF{SU};
my $PS = $CONF{PS};
my $GREP = $CONF{GREP};
my $VGREP = $CONF{VGREP};
my $KILL = $CONF{KILL};
my $PSPLAIN = $CONF{PSPLAIN};
my $AWK = $CONF{AWK};
my $HATYPE = "$VCSHOME/bin/hatype";
my $HARES = "$VCSHOME/bin/hares";
my $HAGRP = "$VCSHOME/bin/hagrp";
my $HANOTIFY = "$VCSHOME/bin/hanotify";


################# FUNCTION DEFN START ##########################
# Function : check_db2_insthome
# Desc     : Checks if the db2 instance home directory exists
# Input    : $Home 
# Output   : None
# Returns  : (1) -  home does not exist  
#
sub check_db2_insthome  {

	my ($Home) =@_;
	if (! -d "$Home" ) {
		return  1;
	}
	return 0;
}

# Function : check_db2_instowner
# Desc     : Checks if the db2 instance owner exists
# Input    : $InstOwner 
# Output   : None
# Returns  : (1) -  Instance owner does not exist  
#
sub check_db2_instowner  {

	my ($InstOwner) =@_;
	if ( "$InstOwner" eq "" ) {
		return  1;
	}
	return 0;
}

# Function : db2_process_not_exist
# Desc     : Checks if any db2 process db2sysc exists
# Input    : $InstOwner 
# Input    : $Agentdebug 
# Output   : None
# Returns  : (1) - if db2sysc process not exists 
#
sub db2_process_not_exist  {

	my ($InstOwner, $AgentDebug) =@_;
	my $Db2proc = "db2sysc";
	my (@AnyDb2Procs);

	if ( "$InstOwner" ne "" ) {
		@AnyDb2Procs=`$PSPLAIN | $VGREP grep | $GREP "^ *$InstOwner " | $GREP $Db2proc`;
	} else {
		@AnyDb2Procs=`$PSPLAIN | $VGREP grep | $GREP $Db2proc`;
	}

	if (@AnyDb2Procs < 1) {
		if ( $AgentDebug == 1) {
			VCSAG_LOGDBG_MSG (3,"No db2sysc process found.");
		}
		return 1;
	} else {
		if ( $AgentDebug == 1) {
			VCSAG_LOGDBG_MSG (3,"db2sysc process found.");
		}
		return 0;
	}
}

# Function : ValidNodeNum
# Desc     : Checks if the db2 partition number is valid
# Input    : $Nodenum - current database partition number
# Input    : $NumPartitions - total database partitions allowed
# Output   : None
# Returns  : (1) - db2 partition number is valid
#
sub ValidNodeNum {

	my ($Partition_arr, $NodeNum) =@_ ;
	my $ret = 0;
	my ($i);
	my $NumPartitions = $#$Partition_arr;

	VCSAG_LOGDBG_MSG (3,"In ValidNodeNum\(\): NodeNum=$NodeNum, NumPartitions=$NumPartitions");
	for ($i=0; $i<=$NumPartitions; $i++) {
		VCSAG_LOGDBG_MSG (3,"In ValidNodeNum\(\): Partition_arr[$i]=$Partition_arr->[$i]");
		if ($Partition_arr->[$i] == $NodeNum) {
			$ret = 1;
			VCSAG_LOGDBG_MSG (3,"In ValidNodeNum\(\): $NodeNum is OK"); 
			last;
		}
	}
	return ($ret);
}

# Function : GetDBPartitions
# Desc     : Checks the total number of database partitions
# Input    : $insthome 
# Output   : None
# Returns  : @Partitions 
#
sub GetDBPartitions {

	my ($InstHome) =@_;
	my $i = 0;
	my $ConfigFile = sprintf("%s/sqllib/db2nodes.cfg", $InstHome);
	my (@tmp, @Partitions, $line, $fp);

	if ( -e "$ConfigFile") {
		$main::EE = 0;
		if (open($fp, "<$ConfigFile")) {
			while ($line = <$fp>) {
				@tmp = split(/\s+/, $line);
				if (!length($tmp[0])) {
					next;
				} else {
					$Partitions[$i] = $tmp[0];
					$i++;
				}
			}
		}
	}
	return @Partitions;
}

# Function : IsMPPSupportable
# Desc     : Checks if MPP mode is supportable
# Input    : $InstHome 
# Output   : None
# Returns  : (1) -  Support MPP
#
sub IsMPPSupportable  {

	my ($InstHome) =@_;
	my ($num, $line);

	$num = 0;
	if (-e "$InstHome/sqllib/db2nodes.cfg") {
		open(FILE, "< $InstHome/sqllib/db2nodes.cfg");
		while (<FILE>) {
			$line = $_;
			$line =~s/^\s+//;
			$line =~s/\s+$//;
			if ($line ne "") {
				$num = $num + 1;
			}
		}
		close(FILE);
	}
	return $num;
}

# Function : IsEE
# Desc     : Checks if it is EE or EEE
# Input    : $InstHome 
# Output   : None
# Returns  : (1) -  if EE
#
sub IsEE  {

	my ($InstHome) =@_;
	if ( -e "$InstHome/sqllib/db2nodes.cfg" ) {
		return 0;
	}
	return 1;
}

# Function : GetNetName
# Desc     : Get the fast switch netname from db2nodes.cfg
# Input    : $InstHome 
# Input    : $NodeNum 
# Input    : $Hostname 
# Output   : None
# Returns  : $NetName
#
sub GetNetName  {

	my ($InstHome, $NodeNum, $Hostname) =@_;
	my $ConfigFile = sprintf("%s/sqllib/db2nodes.cfg", $InstHome);
	my $NetName = "";
	my (@tmp, $NumCol, $line, $fp);

	if ( -e "$ConfigFile") {
		if (open($fp, "<$ConfigFile")) {
			while ($line = <$fp>) {
				@tmp = split(/\s+/, $line);
				$NumCol=@tmp;
				if (!length($tmp[0])) {
					next;
				} else {
					if ($NumCol == 4) {
						if (("$tmp[0]" eq "$NodeNum") && ("$tmp[3]" ne "$tmp[1]") && ("$tmp[3]" ne "$Hostname")) {
							$NetName = "$tmp[3]";
						}
					}
				}
			}
		}
	}
	chomp($NetName);
	return $NetName;
}


# Function : db2gcf_mpp_startup
# Desc     : Startup DB2 for mpp environment
# Input    : $InstHome 
# Input    : $InstOwner 
# Input    : $NodeNum 
# Input    : $Parms 
# Input    : $AgentDebug 
# Input    : $Encoding 
# Output   : None
# Returns  : (0) if db2 starts successfully
#
sub db2gcf_mpp_startup  {

	my ($InstHome, $InstOwner, $CSH, $NodeNum, $Parms, $AgentDebug, $Encoding) =@_;
	my ($ENCODE, $DBAMGR, $DBAMGRSTATUS, $DB2CMD, $RetryLimit, $out, $rc);
	my $ret = 1;

	$DBAMGR = "$InstHome/sqllib/bin/db2gcf -u -i $InstOwner -p $NodeNum";
	$DBAMGRSTATUS = "$InstHome/sqllib/bin/db2gcf -s -i $InstOwner -p $NodeNum";
		
	$RetryLimit = 20;
	while ($RetryLimit) {
		if ("$CSH" eq "true") {
			$DB2CMD="$SU $InstOwner -c \'source $InstHome/sqllib/db2cshrc; $DBAMGR\'";
		} else {
			$DB2CMD="$SU $InstOwner -c \'. $InstHome/sqllib/db2profile; $DBAMGR\'";
		}
		$out = `$DB2CMD`;
		if ($?) {
			$RetryLimit = $RetryLimit - 1;
			if ( $AgentDebug == 1) {
				VCSAG_LOGDBG_MSG (3,"Online call db2gcf with $RetryLimit RetryLimit left:");
				$out = concat_db2_msg($out);
				VCSAG_LOGDBG_MSG (3, "Output returned :$out ");
			}	
			sleep (1);
		} else {
			sleep (10);
			$rc=system($DBAMGRSTATUS);
			if ($rc == 0) {
				$ret = 0;
				last;
			}
		}
	}
	$ENCODE = GetEncodeStr($Encoding);
	VCSAG_LOG_MSG ("N", "DB2 startup returned the output: $out", 72, "$ENCODE", "$out");
	return ($ret);
}

# Function : conventional_db2_startup
# Desc     : Startup DB2 using db2start command
# Input    : $InstHome 
# Input    : $InstOwner 
# Input    : $NodeNum 
# Input    : $Parms 
# Input    : $AgentDebug 
# Input    : $Encoding 
# Output   : None
# Returns  : (0) - if db2 starts successfully
#
sub conventional_db2_startup  {

	my ($InstHome, $InstOwner, $CSH, $NodeNum, $Parms, $AgentDebug, $Encoding) =@_;
	my ($out, $ENCODE, $DBAMGR, $DB2CMD, $RetryLimit);
	my $started = 0;
	my $ret = 1;

	$DBAMGR = "$InstHome/sqllib/adm/db2start";
	$RetryLimit = 20;
	while ($RetryLimit) {
		if ("$CSH" eq "true") {
			$DB2CMD="$SU $InstOwner -c \'source $InstHome/sqllib/db2cshrc; $DBAMGR $Parms\'";	
		} else {
			$DB2CMD="$SU $InstOwner -c \'. $InstHome/sqllib/db2profile; $DBAMGR $Parms\'";
		} 
		$out=`$DB2CMD`;
		$started=grep /SQL[1063]|[1026]N/i, $out;
		if ($started > 0) {
			$ret = 0;
			last;	
		}
		$RetryLimit = $RetryLimit - 1;
		if ( $AgentDebug == 1) {
			VCSAG_LOGDBG_MSG (3,"Online call db2start with $RetryLimit RetryLimit left:");
			$out = concat_db2_msg($out);
			VCSAG_LOGDBG_MSG (3, "Output returned :$out ");
		}
		sleep (1);
	}
	$ENCODE = GetEncodeStr($Encoding);
	VCSAG_LOG_MSG ("N", "DB2 startup returned the output: $out", 72, "$ENCODE", "$out");
	return ($ret);
}


# Function : db2gcf_mpp_stop
# Desc     : Stop DB2 for mpp environment using db2gcf command
# Input    : $InstHome 
# Input    : $InstOwner 
# Input    : $NodeNum 
# Input    : $Parms 
# Input    : $AgentDebug 
# Input    : $Encoding 
# Output   : None
# Returns  : (0) if db2 stop is successful 
#
sub db2gcf_mpp_stop  {

	my ($InstHome, $InstOwner, $CSH, $NodeNum, $Parms, $AgentDebug, $Encoding) =@_;
	my ($ENCODE, $DBAMGR, $DB2CMD, $RetryLimit, $out);
	my $ret = 1;

	$DBAMGR = "$InstHome/sqllib/bin/db2gcf -d -i $InstOwner -p $NodeNum";
	$RetryLimit = 20;
	while ($RetryLimit) {
		if ("$CSH" eq "true") {
			$DB2CMD="$SU $InstOwner -c \'source $InstHome/sqllib/db2cshrc; $DBAMGR\'";
		} else {
			$DB2CMD="$SU $InstOwner -c \'. $InstHome/sqllib/db2profile; $DBAMGR\'";
		}
		$out = `$DB2CMD`;
		if ($?) {
			$RetryLimit = $RetryLimit - 1;
			if ( $AgentDebug == 1) {
				VCSAG_LOGDBG_MSG (3,"Offline call db2gcf with $RetryLimit RetryLimit left:");
				$out = concat_db2_msg($out);
				VCSAG_LOGDBG_MSG (3, "Output returned :$out ");
			}
			sleep (1);
		} else {
			$ret = 0;
			last;
		}
	}
	$ENCODE = GetEncodeStr($Encoding);
	VCSAG_LOG_MSG ("N", "DB2 stop returned the output: $out", 69, "$ENCODE", "$out");
	return ($ret);
}

# Function : conventional_db2_stop
# Desc     : Stop DB2 for using db2stop command
# Input    : $InstHome 
# Input    : $InstOwner 
# Input    : $NodeNum 
# Input    : $Parms 
# Input    : $AgentDebug 
# Input    : $Encoding 
# Output   : None
# Returns  : (0) - if db2 stops successfully
#
sub conventional_db2_stop  {

	my ($InstHome, $InstOwner, $CSH, $NodeNum, $Parms, $AgentDebug, $Encoding) =@_;
	my ($out, $ENCODE, $DBAMGR, $DB2CMD, $RetryLimit);
	my $stopped = 0;
	my $ret = 1;

	$DBAMGR = "$InstHome/sqllib/adm/db2stop force";
	$RetryLimit = 20;
	while ($RetryLimit) {
		if ("$CSH" eq "true") {
			$DB2CMD="$SU $InstOwner -c \'source $InstHome/sqllib/db2cshrc; $DBAMGR $Parms\'";	
		} else {
			$DB2CMD="$SU $InstOwner -c \'. $InstHome/sqllib/db2profile; $DBAMGR $Parms\'";
		} 
		$out=`$DB2CMD`;
		$stopped=grep /SQL1064N/i, $out;
		if ($stopped > 0) {
			$ret = 0;
			last;	
		}
		$RetryLimit = $RetryLimit - 1;
		if ( $AgentDebug == 1) {
			VCSAG_LOGDBG_MSG (3,"Offline call $DBAMGR $Parms with $RetryLimit RetryLimit left:");
			$out = concat_db2_msg($out);
			VCSAG_LOGDBG_MSG (3, "Output returned :$out ");
		}
		sleep (1);
	}
	$ENCODE = GetEncodeStr($Encoding);
	VCSAG_LOG_MSG ("N", "DB2 shutdown returned the output: $out", 74, "$ENCODE", "$out");
	return ($ret);
}

# Function : activate_db
# Desc     : activate the database
# Input    : $InstHome 
# Input    : $InstOwner 
# Input    : $DBName 
# Input    : $AgentDebug 
# Input    : $Encoding 
# Output   : None
# Returns  : (0) - if the command executed successfully
#
sub activate_db  {

	my ($InstHome, $InstOwner, $CSH, $DBName, $AgentDebug, $Encoding) =@_;
	my ($ENCODE, $DBAMGR, $DB2CMD, $RetryLimit, $ret, $out, $status);

	$DBAMGR = "$InstHome/sqllib/bin/db2 activate db $DBName";
	if ("$CSH" eq "true") {
		$DB2CMD="$SU $InstOwner -c \'source $InstHome/sqllib/db2cshrc; $DBAMGR\'";	
	} else {
		$DB2CMD="$SU $InstOwner -c \'. $InstHome/sqllib/db2profile; $DBAMGR\'";
	} 
	$out=`$DB2CMD`;
	$status=grep /DB20000I/i, $out;
	if ($status > 0) {
		$ret = 0;
	} else {
		VCSAG_LOG_MSG ("W", "Cannot activate the database $DBName", 68, "$DBName");
		$ret = 1;
	}

	$out = concat_db2_msg($out);
	$ENCODE = GetEncodeStr($Encoding);
	VCSAG_LOG_MSG("N", "db2 activate db command returned the output: $out", 67, "$ENCODE", "$out");	
	return ($ret);
}

# Function : get_db2gcf_status
# Desc     : find out the status of the partition using db2gcf
# Input    : $InstHome 
# Input    : $InstOwner 
# Input    : $NodeNum 
# Output   : None
# Returns  : (0) - if the command executed successfully
#
sub get_db2gcf_status  {

	my ($InstHome, $InstOwner, $NodeNum) =@_;
	my ($DB2CMD, $ret);

	$DB2CMD = "$InstHome/sqllib/bin/db2gcf -s -i $InstOwner -p $NodeNum > /dev/null";
	$ret = system($DB2CMD);
	return ($ret);
}

# Function : check_db2_pid
# Desc     : find out the status of the db2 pid
# Input    : $PidFile 
# Input    : $AgentDebug 
# Output   : None
# Returns  : (0) - if the pid is still alive
#
sub check_db2_pid  {

	my ($PidFile, $AgentDebug) =@_;
	my $ret = 1;
	my $pid = "";

	$pid = `$CAT $PidFile`;
	chomp($pid);
	if ( "$pid" eq "" ) {
		if ( $AgentDebug == 1) {
			VCSAG_LOGDBG_MSG (3, "No pid found in the file $PidFile.");
		}
		return ($ret);
	}
	`$KILL -0 $pid 2> /dev/null`;
	$ret = $?;
	if ( $AgentDebug == 1) {
		VCSAG_LOGDBG_MSG (3, "The return code is $ret from check_db2_pid\(\) for the process $pid");
	}
	return ($ret);
}

# Function : update_switchname
# Desc     : update the switchname in the DB2 global registry
# Input    : $InstHome 
# Output   : None
# Returns  : (0) 
#
sub update_switchname  {

	my ($InstHome, $NetName) =@_;
	my ($DBAMGR);

	if ((length($NetName)) && (-x "$InstHome/sqllib/bin/db2greg")) {
		$DBAMGR = "$InstHome/sqllib/bin/db2greg -delvarrec variable=switchname 2> /dev/null > /dev/null; ";
		$DBAMGR = $DBAMGR . "$InstHome/sqllib/bin/db2greg -addvarrec variable=switchname,value=$NetName; sleep 3; ";
		system($DBAMGR);
	} 
	return (0);
}

# Function : concat_db2_msg
# Desc     : concatenate multiple lines of DB2 output into a single line
# Input    : $output 
# Output   : none
# Returns  : $out
#
sub concat_db2_msg {

	my ($output) =@_;
	my ($BEGLINE, $ENDLINE, $line);
	
	$BEGLINE="\n+-----------------------------------------------------------" .
                                                        "---------+\n";
	$ENDLINE="+===========================================================" .
                                                        "=========+\n";
	$line .= $BEGLINE;
	$line .= $output;
	$line .= $ENDLINE;
	return $line;
}

# Function : get_sql_code
# Desc     : get the DB2 error code
# Input    : $out 
# Output   : none
# Returns  : $db2errcode

sub get_sql_code {

	my ($out, $AgentDebug)=@_ ;
	my ($db2errcode, @out1, $line, $string, $str);
	my (@substr ,@value);


	@out1 = split(/\n/, $out);
	foreach $line (@out1) {
		chomp $line;
		if ($line =~/SQL[0-9]{4}[A-Z]/) {
			$string = $line;
			@substr = split (/ /,$string);
			foreach my $str(@substr) {
				$str =~ s/^\s+//;
				chomp($str);
				if ($str =~/^SQL[0-9]{4}[A-Z]$/) {
					$db2errcode = $str;
					last;
				}
			}
			last;
		}
	}
	if ( $AgentDebug == 1) {
		VCSAG_LOGDBG_MSG (3, "get_sql_code\(\) returns $db2errcode");
	}
	return $db2errcode;
}

# Function : db2_error_procedure
# Desc     : checking and handling DB2 error code
# Input    : $ResName 
# Input    : $db2errcode 
# Input    : $AgentDebug 
# Output   : none
# Returns  : $state
#
sub db2_error_procedure {

	my($ResName, $db2errcode, $AgentDebug, $AgentDirectory)=@_ ;
	my $db2errfile="$AgentDirectory/db2error.dat";
	my $action="FAILOVER";
	my (%Db2_err_table, @fields, $line, $state, $ret);

	if ( $AgentDebug == 1) {
		VCSAG_LOGDBG_MSG (3, "Inside the function db2_error_procedure\(\): Checking the error code $db2errcode ...");
	}
	if  ( -f $db2errfile && -T $db2errfile ) {
		if ( $AgentDebug == 1) {
			VCSAG_LOGDBG_MSG (3, "Found the file $db2errfile");
		}
		open (DATA, "$db2errfile");
		while (<DATA>) {
			if ($_ =~ /^\s+/) {
				s/^\s+//g;
			}
			if ($_ =~ /:/) {
			if ( $AgentDebug == 1) {
				VCSAG_LOGDBG_MSG (3, "At least one item in $db2errfile");
			}
				$line = $_;
				@fields = split(/:/, $line);
				if ($fields[0] ne "") {
					$Db2_err_table{uc($fields[0])} = uc($fields[1]);
					if ( $AgentDebug == 1) {
						VCSAG_LOGDBG_MSG (3, "At least one item in the hash table");
					}
				}
			}
		}
		close(DATA);
		if (exists $Db2_err_table{uc($db2errcode)}) {
			$action=$Db2_err_table{uc($db2errcode)};	
			chomp ($action);
			if ( $AgentDebug == 1) {
				VCSAG_LOGDBG_MSG (3, "Found the db2 error code $db2errcode in $db2errfile");
				VCSAG_LOGDBG_MSG (3, "Action returned is $action");
			}
		}
	}

	if ("$action" eq "FAILOVER") {
		$state = 100;
	} elsif ("$action" eq "UNKNOWN") {
		db2_error_notify($ResName, $action, $db2errcode);
		$state = 99;
	} elsif ("$action" eq "WARN") {
		db2_error_notify($ResName, $action, $db2errcode);
		$state = 110;
	} elsif ("$action" eq "NOFAILOVER") {
		$ret = freeze_service_group($ResName);
		db2_error_notify($ResName, $action, $db2errcode);
		$state = 100;
	} else {
		$state = 110;
	}
	if ( $AgentDebug == 1) {
		VCSAG_LOGDBG_MSG (3, "Return final exit code $state");
	}
	return $state;
}

# Function : GetSleepTime
# Desc     : Calulate the sleep time for the clean script to complete
# Input    : none
# Output   : none
# Returns  : $sleeptime
#
sub GetSleepTime {
	my ($MI, $TL, $CT, $MaxSleepTime, $DefSleepTime);

	$MI=`$HATYPE -display Db2udb| $GREP " MonitorInterval" | $AWK '{print \$3}' 2>/dev/null`;
	$TL=`$HATYPE -display Db2udb| $GREP ToleranceLimit | $AWK '{print \$3}' 2>/dev/null`;
	$CT=`$HATYPE -display Db2udb| $GREP CleanTimeout | $AWK '{print \$3}' 2>/dev/null`;
	$MaxSleepTime=$CT - 90;
	$DefSleepTime=((1+$TL) * $MI);

	if ($MaxSleepTime < 1) {
		return (0);
	}
	else {
		if ($DefSleepTime <= $MaxSleepTime) {
			return ($DefSleepTime);
		}
		else {
			return ($MaxSleepTime);
		}
	}	
}

# Function : custom_script
# Desc     : Execute the script start_custom_${InstOwner}_${NodeNum}
# Desc     : or the stop_custom_${InstOwner}_${NodeNum}
# Desc     : or the monitor_custom_${InstOwner}_${NodeNum}
# Input    : $Action - one of these 3 choices: startup, shutdown, or monitor
# Input    : @ARGV
# Output   : None
# Returns  : $ret - return code 
#
sub custom_script {

	my ($Action, $ResName, $InstOwner, $InstHome, $IndepthMonitor, $DBName, $NodeNum, $StartUpOpt, $ShutDownOpt, $AgentDebug, $Encoding, $WarnOnlyIfDBQueryFailed, $LastWarningDay, $ContainerName) = @_;
	my ($Script, $AgentDirectory, $out, $ret, $sqlerrcode, $ENCODE, $arg, @arglist);

	shift;
	#VCSAG_LOGDBG_MSG (3,"Argument passed to the custom script:");
        #VCSAG_LOGDBG_MSG (3,"Action=$Action");
        #VCSAG_LOGDBG_MSG (3,"ResName=$ResName");
        #VCSAG_LOGDBG_MSG (3,"InstOwner=$InstOwner");
        #VCSAG_LOGDBG_MSG (3,"InstHome=$InstHome");
        #VCSAG_LOGDBG_MSG (3,"IndepthMonitor=$IndepthMonitor");
        #VCSAG_LOGDBG_MSG (3,"DBName=$DBName");
        #VCSAG_LOGDBG_MSG (3,"NodeNum=$NodeNum");
        #VCSAG_LOGDBG_MSG (3,"StartUpOpt=$StartUpOpt");
        #VCSAG_LOGDBG_MSG (3,"ShutDownOpt=$ShutDownOpt");
        #VCSAG_LOGDBG_MSG (3,"AgentDebug=$AgentDebug");
        #VCSAG_LOGDBG_MSG (3,"Encoding=$Encoding");
        #VCSAG_LOGDBG_MSG (3,"WarnOnlyIfDBQueryFailed=$WarnOnlyIfDBQueryFailed");
        #VCSAG_LOGDBG_MSG (3,"LastWarningDay=$LastWarningDay");
        #VCSAG_LOGDBG_MSG (3,"ContainerName=$ContainerName");

	$AgentDirectory=dirname($0);
	$ret = 0;
	if ( "$Action" eq "startup" ) {
		$Script="$AgentDirectory/start_custom_${InstOwner}_${NodeNum}";
	} elsif ( "$Action" eq "shutdown" ) {
		$Script="$AgentDirectory/stop_custom_${InstOwner}_${NodeNum}";
	} elsif ( "$Action" eq "monitor" ) {
		$Script="$AgentDirectory/monitor_custom_${InstOwner}_${NodeNum}";
		$ret = 100;
	} else {
		VCSAG_LOG_MSG("E", "Invalid action $Action for the custom script.", 73, "$Action");
		return ($ret);
	}

	if  ( -f $Script ) {
		for $arg (@_) {
			push(@arglist, $arg);
		}
		$out=`$Script @arglist`;
		$ret = $? >> 8;
		if (("$Action" ne "monitor") || ($AgentDebug == 1)) {
			$out = concat_db2_msg($out);
			$ENCODE = GetEncodeStr($Encoding);
			VCSAG_LOG_MSG("N","DB2 custom $Action from $Script returned the output: $out",21, "$ENCODE", "$Action", "$Script", "$out");
		}
		if (("$Action" eq "monitor") && ($ret != 110)) {
			$sqlerrcode = get_sql_code($out, $AgentDebug);
			if ($WarnOnlyIfDBQueryFailed != 1) {
				VCSAG_LOGDBG_MSG (3, "WarnOnlyIfDBQueryFailed is not set to 1 and SQL error returned is $sqlerrcode, thus calling db2_error_procedure() to checking this error code");
				$ret = db2_error_procedure($ResName, $sqlerrcode, $AgentDebug, $AgentDirectory);
				VCSAG_LOGDBG_MSG (3, "return code returned from db2_error_procedure() is $ret");
			} else {
				$ret = 110;
			}
		}
	} else {
		VCSAG_LOG_MSG("W","Custom $Action script $Script does not exist.", 71, "$Action", "$Script"); 
		if ("$Action" eq "monitor") {
			$ret = 110;
		}
		return ($ret);
	}
	return ($ret);	
}

# Function : kill_db2_processes
# Desc     : kill the db2sysc processes if exist
# Input    : $InstOwner 
# Input    : $InstHome 
# Input    : $NodeNum 
# Input    : $AgentDebug 
# Output   : None
# Returns  : None
#
sub kill_db2_processes  {

	my ($InstOwner, $InstHome, $NodeNum, $AgentDebug) =@_;
	my ($pidfile, $pid_to_mon, $CMD, $PSCMD, $numfields, @fields); 
	my $printmsg = 1;
	
	$pidfile = sprintf("%s/sqllib/ctrl/db2pgrp.%03d", $InstHome, $NodeNum);
	if ( -e "$pidfile" ) {
		$pid_to_mon="";
		$pid_to_mon=`$CAT $pidfile`;
		chomp($pid_to_mon);
		if ( $pid_to_mon > 0 ) {
			$CMD="$KILL -9 $pid_to_mon";
			VCSAG_LOG_MSG ("W", "Killing the processes ID $pid_to_mon", 65, "$pid_to_mon");	
			system($CMD);
		}
	}

	$PSCMD="$PSPLAIN | $GREP \"\^ *$InstOwner \"";
	open(PROCLIST, "$PSCMD |");
	while (<PROCLIST>) {
		chomp;
		@fields = split(/\s+/, $_);
		$numfields = @fields;
		if (($fields[0] eq $InstOwner) && ($fields[$numfields-1] =~ m/^db2/)) {
			if ( $printmsg == 1 ) {
				VCSAG_LOG_MSG ("W", "Attempting to kill all processes owned by $InstOwner", 60, "$InstOwner");	
				$printmsg = 0;
			}
			$CMD="$KILL -9 $fields[1]";
			system($CMD);
		}
	}
}

# Function : GetEncodeStr
# Desc     : halog will display the output from third party (in this case, DB2)
#            using the user specified encoding scheme
# Input    : $Encoding 
# Output   : None
# Returns  : $str
#
sub GetEncodeStr  {

	my ($Encoding) =@_;
	my $str = "";

	if ($Encoding) {
		$str = "-encoding $Encoding";
	}
	return $str;
}

# Function : freeze_service_group
# Desc     : freeze the service group 
# Input    : $resname 
# Output   : None
# Returns  : (0) if success
#
sub freeze_service_group  {

	my ($ResName) =@_;
	my $sgname = "";
	my (@tmp, $ret);

	$ret = 1;
	open (OUT, "$HARES -display $ResName -attribute Group 2> /dev/null |");
	while (<OUT>) {
		@tmp = split;
		if ("$tmp[0]" eq "$ResName") {
			$sgname = "$tmp[3]";
			last;
		} else {
			next;
		}
	}

	if ("$sgname" ne "") {
		VCSAG_LOGDBG_MSG (3,"hares -display $ResName command is successful. Service group name is $sgname ");
	} else {
		VCSAG_LOGDBG_MSG (3,"hares -display $ResName command failed to find the service group name ");
	}
			
	if ("$sgname" ne "") {
		`$HAGRP -freeze $sgname`;
		if ($? == 0) {
			$ret = 0;
			VCSAG_LOGDBG_MSG (3,"Service group $sgname is frozen");
		} else {
			VCSAG_LOGDBG_MSG (3,"Service group $sgname cannot be frozon");
		}
	}
	return $ret;
}

# Function : db2_error_notify
# Desc     : handle notifications using hanotify utility
# Input    : $resname 
# Input    : $action 
# Input    : $db2errcode 
# Output   : None
# Returns  : (0) if success
#
sub db2_error_notify  {

	my ($ResName, $action, $db2errcode) =@_;
	my $NOTIFYID = "1.3.6.1.4.1.1302.3.8.10.1.2.8.4";
	my $ret = 1;

	`$HANOTIFY -i $NOTIFYID -l Error -n "DB2 failure" -T 7 -t "Failure in DB2 indepth monitoring" -o 4 -c $ResName -C 7 -m "DB2 error code $db2errcode in indepth monitoring. Resource action is $action"`;
	if ($? == 0) {
		$ret = 0;
		VCSAG_LOGDBG_MSG (3,"Notification successful. Error code notified: $db2errcode, State: $action");
	} else {
		VCSAG_LOGDBG_MSG (3,"hanotify command failed");
	}
	return $ret;
}

1;
